
// FaceAnimator.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, July 2005

/* FaceAnimator is a panel containing a face image of
   Prof Bob and a separate mouth image which is drawn over 
   the face.

   A timer is started which ticks away, first offseting the
   mouth downwards, then drawing it back at its normal 
   position. This gives the impression of Prof Bob talking.

   The timer is started (or resumed) by the top-level ProfBob
   calling startAnimation(), and is paused by stopAnimation().
   These methods are called to coincide with the start and
   end of speech synthesis. Thus, Prof Bob seems to talk
   as his response to the user is being spoken.

   The animation will only stop when the mouth is back in its
   default position, with no offset. This means there may be a
   short delay between the call to stopAnimation() and the 
   mouth actually stopping.

   The mouth's offset is randomly generated, and doesn't bear 
   any relationship to what's actually being said. Surprisingly, 
   this isn't particularly noticable.
*/

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Random;


public class FaceAnimator extends JPanel
                          implements ActionListener 
{
  private static final int DELAY = 100;  // in ms, for the timer

  // normal position for the mouth
  private static final int MOUTH_X = 39;
  private static final int MOUTH_Y = 101; 

  // the offset downwards will be between 1 and MAX_OFFSET+1
  private static final int MAX_OFFSET = 2; 


  private ImageIcon faceIm, mouthIm;   // the face and mouth images
  private Timer animTimer = null;     // the timer for controlling the animation

  // for controlling the mouth movement up and down
  private boolean isOffset = false;
  private int offset = 0;    // offset down from the mouth's normal posn
  private Random rand;

  private boolean stopAnimation = false;  // to pause the timer


  public FaceAnimator()
  {
    faceIm = new ImageIcon("face.jpg");    // load the images
    mouthIm = new ImageIcon("mouth.jpg");
    rand = new Random();

    // make the panel the same size as the face image
    Dimension d = new Dimension(faceIm.getIconWidth(), faceIm.getIconHeight());
    setMinimumSize(d);
    setPreferredSize(d);
  } // end of FaceAnimator()


  public void paintComponent(Graphics g)
  /* Draw the face and the mouth. The mouth will be offset downwards
     by some amount from its 'normal' (MOUTH_X, MOUTH_Y) location. */
  {
    super.paintComponent(g);
    faceIm.paintIcon(this, g, 0, 0);
    mouthIm.paintIcon(this, g, MOUTH_X, MOUTH_Y + offset);
  } // end of paintComponent()


  public void actionPerformed(ActionEvent e)
  /* Called by the timer. Update the mouth offset and redraw the image. 
     The animation may be stopped, but only if the mouth is 
     currently not offset down.
  */
  {  
    if (stopAnimation && !isOffset)   // mouth is not offset, so can stop
      animTimer.stop();    // pause the timer
    else {
      offset = getMouthOffset();
      repaint();  
    }
  } // end of actionPerformed()


  public void startAnimation()
  // ProfBob calls this to start or resume the animation
  {
    stopAnimation = false;
    if (animTimer == null) {
      animTimer = new Timer(DELAY, this);   // start the timer
      animTimer.start();
    }
    else if (!animTimer.isRunning())
      animTimer.restart();    // resume the timer
  } // end of startAnimation()


  public void stopAnimation()
  // ProfBob calls this to request that the animation be stopped
  {  stopAnimation = true;  }


  private int getMouthOffset()
  /* If the mouth is not currently offset, then calculate an offset.
     If the mouth is offset, then return a 0 offset so it's drawn
     back at it's original position.
  */
  { int offset;
    if (!isOffset) {  // not offset, so can move the mouth down
      offset = rand.nextInt(MAX_OFFSET) + 1;
      isOffset = true;
    }
    else {   // the mouth is offset, so draw it back at its original posn
      offset = 0;
      isOffset = false;
    }
    return offset;
  }  // end of getMouthOffset()

}  // end of FaceAnimator class
