
// KeyResponses.java
// Andrew Davison, July 2005, ad@fivedots.coe.psu.ac.th

/* KeyResponses stores a single group of keywords and their
   associated response lines. These were loaded by the Responder
   class and stored here by calls to addKey() and addResponse().

   Responder calls respond() with the user's message. If the
   message contains one of this object's keywords, then one of
   its response lines is returned.

   The response line is selected at semi-random; 
   semi-random because the line will not be the same as 
   the one returned by the previous call to respond().

   A response line may contain a '$', in which case the keyword
   that triggered the match is inserted in place of the '$' before 
   the response is returned.
*/

import java.util.*;


public class KeyResponses
{
  // keywords and response lines managed by this object
  private ArrayList keys;
  private ArrayList responses;

  // used in the semi-random selection of a response
  private Random rand;
  private int previousRandInt;


  public KeyResponses()
  {
    keys = new ArrayList();
    responses = new ArrayList();
    rand = new Random();
    previousRandInt = -1;
  } // end of KeyResponses();


  public void addKey(String keyStr)
  // called by Responder to add a key
  {  
    // System.out.println("New key: " + keyStr);
    keys.add(keyStr);  
  }

  public void addResponse(String respStr)
  // called by Responder to add a response
  { 
    // System.out.println("New response: " + respStr);
    responses.add(respStr);  
  }


  public String respond(String msg)
  /* Responder calls respond() with the user's message; if a 
     keyword is found in the message, then a response line is 
     selected, modified, and returned. If no keyword is found, 
     then null is returned. */
  {
    String key;
    if ((key = findKeyword(msg)) != null)    // found a keyword?
      return buildResponse(key);    // build and return a response
    else
      return null;
  }  // end of respond()


  private String findKeyword(String msg)
  /* Search the msg for a keyword, returning the matching
     keyword, or null otherwise. */
  {
    String lowerMsg = msg.toLowerCase();
    String[] tokens = lowerMsg.split(" ");

    // check each token in the message to see if it's a keyword
    String key;
    for (int i=0; i < tokens.length; i++)
      for (int j=0; j < keys.size(); j++) {
        key = (String) keys.get(j);
        if (tokens[i].startsWith(key)) {
          // System.out.println("Found key word: " + key);
          return key;
        }
      }
    // System.out.println("No key word found");
    return null;
  }  // end of findKeyword()


  private String buildResponse(String key)
  /* Select a response line at semi-random from the responses
     ArrayList. If it contains a '$', then replace the '$' with 
     the keyword.
  */
  {
    int posn = getRandomInt( responses.size() );
    String str = (String) responses.get(posn);
    int idx = str.indexOf('$');
    if (idx == -1)  // no '$' found
      return str;    // return response unchanged
    else {   // replace '$' by the keyword
      String resp = str.substring(0, idx) + key + 
                    str.substring(idx+1);
      // System.out.println("kr resp: " + resp);
      return resp;
    }
  } // end of buildResponse()


  private int getRandomInt(int max)
  // avoid getting the same random integer as last time
  {
     int num = rand.nextInt(max);
     while (num == previousRandInt)
       num = rand.nextInt(max);
     previousRandInt = num;
     return num;
  } // end of getRandomInt()

} // end of KeyResponses class

