
// ProfBob.java
// Andrew Davison, July 2005, ad@fivedots.coe.psu.ac.th

/* ProfBob is a mildly amusing example of how to use speech synthesis,
   as supported by the freeTTS implementation of the Java Speech API
   (JSAPI).
      JSAPI: http://java.sun.com/products/java-media/speech/
      freeTTS: http://freetts.sourceforge.net

   The user enters a query into a text field, and Prof Bob speaks
   a suitable response, and the reponse text is shown in another
   textfield. 

   A face image (supposedly of Prof Bob) is animated while the 
   response is being spoken.
*/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;



public class ProfBob extends JFrame implements ActionListener
{
  // GUI data structures
  private JTextField jtfMsg;     // for the user's input
  private JTextField jtfResp;    // for ProfBob's textual response

  private FaceAnimator faceAnim;  // Prof Bob's animated face
  private Responder responder;    // the object that generates a textual response
  private Speaker speaker;        // the synthesizer that speaks the response
  private String response;


  public ProfBob()
  {
    super("Prof. Bob answers your questions");

    initializeGUI();
    responder = new Responder();
    speaker = new Speaker(this);

    addWindowListener( new WindowAdapter() {
      public void windowClosing(WindowEvent e)
      { speaker.closeDown();    // close down the synthesizer
        System.exit(0);
      }
    });

    pack();
    setResizable(false);
    jtfMsg.requestFocusInWindow();   // give the input text field the focus
    setVisible(true);
  } // end of ProfBob();


  private void initializeGUI()
  /* The GUI is made up of an anmated face image panel, an output 
     textfield for Prof Bob's textual response, and an input textfield 
     for the user, all in a vertical box layout
  */
  {
    Container c = getContentPane();
    c.setLayout( new BoxLayout(c, BoxLayout.Y_AXIS) );   // vertical layout

    // animated face image panel
    faceAnim = new FaceAnimator();
    JPanel animP = new JPanel( new FlowLayout() );
    animP.add(faceAnim);
    c.add(animP);

    // textfield for Prof Bob's response
    JLabel jlResp = new JLabel("Response: ");
    jtfResp = new JTextField(25);
    jtfResp.setEditable(false);

    JPanel outputP = new JPanel( new FlowLayout() );
    outputP.add(jlResp); outputP.add(jtfResp);
    c.add(outputP);

    // textfield for the user's input
    JLabel jlMsg = new JLabel("Message: ");
    jtfMsg = new JTextField(25);
    jtfMsg.addActionListener(this); 

    JPanel inputP = new JPanel( new FlowLayout() );
    inputP.add(jlMsg); inputP.add(jtfMsg);
    c.add(inputP);
  }  // end of initializeGUI()


  public void actionPerformed(ActionEvent e)
  /* Pass the user's input message to responder for a suitable
     response, then pass the response to the speaker. 

     When the speaker starts saying the response, it will call
     speechStarted(); when it finishes speaking, it will call
     speechEnded().
  */
  { 
    if (e.getSource() == jtfMsg) {
      String msg = jtfMsg.getText().trim();   // get the user's message
      if (!msg.equals("")) {
        jtfMsg.setEnabled(false);             // prevent any further input
        response = responder.respond(msg);    // get a textual response
        // System.out.println("> " + msg);
        // System.out.println("  " + resp);
        speaker.say(response);    // ask the speaker to say the response
      }
    }
  }  // end of actionPerformed()


  // ----------- methods called by the Speaker --------------

  public void speechStarted()
  // called by the Speaker when it starts saying the response
  { 
    jtfResp.setText(response);   // show the texual response
    faceAnim.startAnimation();   // animate the face
  }  // end of speechStarted()


  public void speechEnded()
  // called by the Speaker when the speech has finished
  {
    faceAnim.stopAnimation();    // stop the animation
    jtfMsg.setText("");          // clear the response field
    jtfMsg.setEnabled(true);     // prepare for new input
    jtfMsg.requestFocusInWindow();
  }  // end of speechEnded()


  // ------------------------------------

  public static void main(String args[]) 
  {  new ProfBob();  }

} // end of ProfBob class
