
// Responder.java
// Andrew Davison, July 2005, ad@fivedots.coe.psu.ac.th

/* Responder's respond() method is called by the top-level
   ProfBob to generate a suitable response for the user's message.

   Responder utilizes two approaches: first it passes the message
   to a series of KeyResponses objects (stored in the keyResps
   ArrayList). Each object in turn looks for a keyword, and if one 
   is found, the object generates a response.

   If none of the KeyResponses objects can find a keyword then
   Responder calls on the VagueResponses object, vagues, which
   serves up a response that is independent of the input message.
   Consequently, the response is rather 'vague'.

   The ArrayList of KeyResponses objects is created by Responder 
   reading in lines from KR_FNM, which consists of groups of 
   keywords and responses delimited by #K and #R markers.

   For example, keyResponses.txt contains:

      #K
      java c++ c# j2me j2ee perl php math 

      #R
      $ is a tool, much like a hammer.
      Learning $ is just the first step.
        :   // more lines
      Try typing $ into Google; it's what I do.

      #K
      exercise lab question exam midterm
      program homework project

      #R
      Why do you hate the $?
      I hope you're not working on the $ with other students?

   Each group of keyword and response lines is used to create a new
   KeyResponses object.
*/

import java.util.*;
import java.io.*;


public class Responder
{
  // data files
  private static final String KR_FNM = "keyResponses.txt";
  private static final String VAGUE_FNM = "vagueResponses.txt";

  private VagueResponses vagues;
  private ArrayList keyResps;     // ArrayList of KeyResponses objects


  public Responder()
  {
    vagues = new VagueResponses(VAGUE_FNM);   // initialize the vague responses

    // initialize the key/responses
    keyResps = new ArrayList();
    loadKeyRespsFile(KR_FNM);
    System.out.println("No. of key/responses groups: " + keyResps.size());
  } // end of Responder();


  public String respond(String msg)
  // called by ProfBob to get a response for the user's message
  {
    KeyResponses kr;
    String response;
    // try the message with each KeyResponses object
    for(int i=0; i < keyResps.size(); i++) {
      kr = (KeyResponses) keyResps.get(i); 
      response = kr.respond(msg);   // try to find a key/response
      if (response != null)
        return response;
    } 
    // no suitable key/response found
    return vagues.respond();
  }  // end of respond()


  // ----------- load the key/responses information -------------------

  private void loadKeyRespsFile(String fnm)
  /* Load a series of lines from fnm, ignoring blank lines and
     lines starting with "//" (i.e. comment lines). 

     A line is either a sequence of keywords or a response line, 
     depending on if a #K or #R marker was seen previously.

     A #K triggers the creation of a new KeyResponses object,
     which is added to the keyResps ArrayList when the next #K 
     is encountered (or eof is reached).
  */
  { 
    System.out.println("Reading key/responses file: " + fnm);

    boolean loadingKeys = false;
    KeyResponses kr = null;
    try {
      BufferedReader br = new BufferedReader( new FileReader(fnm));
      String line;
      while((line = br.readLine()) != null) {
        if (line.length() == 0)      // ignore a blank line
          continue;
        if (line.startsWith("//"))   // ignore a comment line
          continue;

        if (line.startsWith("#K")) {   // arrived at the keywords
          if (kr != null)
            keyResps.add(kr);   // store previous keyResponses Object
          kr = new KeyResponses(); 
          loadingKeys = true;
        }
        else if (line.startsWith("#R"))  // arrived at the response lines
          loadingKeys = false;
        else {  // read in the line (either keywords or response line)
          if (loadingKeys)
            storeKeys(kr, line);
          else 
            storeResponse(kr, line);
        }
      }
      br.close();
      if (kr != null)
        keyResps.add(kr);   // store keyResponses object
    } 
    catch (IOException e) 
    { System.out.println("Error reading file: " + fnm);
      System.exit(1);
    }
  }  // end of loadKeyRespsFile()


  private void storeKeys(KeyResponses kr, String line)
  /* Extract keywords from the line and add them to 
     the KeyResponses object. */
  {
    String keys = line.toLowerCase();
    String[] tokens = keys.split(" ");
    for (int i=0; i < tokens.length; i++)
      kr.addKey(tokens[i]);
  }  // end of storeKeys()


  private void storeResponse(KeyResponses kr, String line)
  // Add the response to the KeyResponses object.
  {
    if (kr != null) {
      String resp = line.trim();
      if (!resp.equals(""))
        kr.addResponse(resp);
    }
  }  // end of storeResponse()


} // end of Responder class

