
// Speaker.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, July 2005

/* Initialize a freeTTS speech synthesizer using the medium
   quality "kevin16" voice. freeTTS can be obtained from: 
      http://freetts.sourceforge.net

   A test message is spoken by ProfBob calling the say() method.

   Speaker is a SpeakerAdapter, and listens for the start of
   a speech event (with speakableStarted()) and its end event (with
   speakableEnded()). 

   When a speech begins, ProfBob is notified. When the speech 
   finishes, ProfBob is notified again.

   (I wanted to use wordStarted() to do something each time a
    new word was spoken, but freeTTS doesn't support its 
    WORD_STARTED event.)

   The synthesizer is closed down by ProfBob calling closeDown().
 */


import java.io.File;
import java.util.*;
import javax.speech.*;
import javax.speech.synthesis.*;


public class Speaker extends SpeakableAdapter 
{
  private static final String VOICE_NAME = "kevin16";
    // kevin16 is freeTTS's medium quality general-purpose US voice

  private Synthesizer synthesizer;
  private ProfBob bob;              // top-level GUI


  public Speaker(ProfBob b)
  { bob = b;
    initSynthesizer();
  }  // end of Speaker()


  private void initSynthesizer()
  {
    System.out.println("Synthesizer being initialized...");
	try {
      // specify the required synthesizer properties
	  SynthesizerModeDesc desc = new SynthesizerModeDesc(
                null,          // engine name (don't care)
                "general",     // mode name -- general usage
                Locale.US,     // locale
                null,          // running (don't care)
                null);         // voice (none specified yet)

      
	  synthesizer = Central.createSynthesizer(desc);
	  if (synthesizer == null) {
		System.err.println( noSynthMsg() );
		System.exit(1);
	  }

	  synthesizer.allocate();   // allocate the specified resources
	  synthesizer.waitEngineState(Synthesizer.ALLOCATED);

      SynthesizerProperties synProps = synthesizer.getSynthesizerProperties();
      synProps.setVoice( getVoice() );    // set the engine's voice
      synProps.setPitch(100);             // lower pitch (deeper) (male normal is ~130)
      // synProps.setSpeakingRate(100.0f);   // speak slower (normal is ~200)

	  synthesizer.resume();     // get synthesizer ready to speak

      // wait until the synthesizer is ready to speak
	  synthesizer.waitEngineState(Synthesizer.RESUMED);
	  synthesizer.waitEngineState(Synthesizer.QUEUE_EMPTY);
      System.out.println("Synthesizer ready");
	} 
    catch (Exception e) 
    {  System.out.println(e);
       System.exit(1);  
    }
  }  // end of initSynthesizer()


  private Voice getVoice()
  // get the Voice object associated with the VOICE_NAME name
  {
    // get the properties for this synthesizer engine
    SynthesizerModeDesc desc = 
          (SynthesizerModeDesc) synthesizer.getEngineModeDesc();
    Voice[] voices = desc.getVoices();

    // check if VOICE_NAME is a known Voice for this engine
    Voice voice = null;
    for(int i = 0; i < voices.length; i++) {
      if (voices[i].getName().equals(VOICE_NAME)) {
        voice = voices[i];
        break;
      }
    }
    if (voice == null) {
      System.err.println("Synthesizer could not find the " +
                          VOICE_NAME + " voice");
      System.exit(1);
    }

    // make the voice older
    // System.out.println("Voice age: " + voice.getAge());
    voice.setAge(Voice.AGE_OLDER_ADULT); // 60+    // AGE_CHILD, < 12
    // System.out.println("New voice age: " + voice.getAge());

    return voice;
  } // end of getVoice()


  private String noSynthMsg() 
  /* Report the "speech.properties" file location as a possible
     source of problems  */
  {
    String message =
         "No synthesizer created.  This may be due to a missing\n" +
         "\"speech.properties\" file that should be in: \n\n";

    message += "   user.home    : " + System.getProperty("user.home") + "\n";
    message += "or java.home/lib: " + System.getProperty("java.home") +
	    File.separator + "lib\n";

    return message;
  } // end of noSynthMsg()


  public void say(String msg)
  // request that msg be spoken; don't wait for it to be carried out
  {
    try {
	  synthesizer.speakPlainText(msg, this);  // add msg to the speaking queue
	} 
    catch (Exception e) 
    {  System.out.println(e); }
  }  // end of sayMessage()


  // -------------- SpeakableAdapter methods ------------

  public void speakableStarted(SpeakableEvent e)
  // called when the synthesizer starts saying the msg
  {  
    // System.out.println("Speech started");
    bob.speechStarted();  
  }       // tell ProfBob


  public void speakableEnded(SpeakableEvent e)
  // called when the synthesizer has finished saying the msg
  {  bob.speechEnded();  }         // tell ProfBob


  // ----------------------------------------------------

  public void closeDown()
  // close down the synthesizer
  {
    try {
      synthesizer.cancel();        // cancel any speaking
	  synthesizer.deallocate();    // free the engine's resources
	} 
    catch (Exception e)
    {  System.out.println(e); }
  }  // end of closeDown()


} // end of Speaker class
